package pw.hamzantal.shopreborn

import org.bukkit.Bukkit
import org.bukkit.Material
import org.bukkit.entity.Player
import org.bukkit.event.inventory.ClickType.*
import org.bukkit.event.inventory.InventoryClickEvent
import org.bukkit.event.inventory.InventoryCloseEvent
import org.bukkit.inventory.Inventory
import org.bukkit.inventory.ItemStack
import kotlin.math.max
import kotlin.math.min

enum class PurchaseType { BUY, SELL }

class PurchaseEvent(
    val inv: Inventory,
    val item: ItemStack,
    val shop: ShopConfig,
    val p: Player,
    val block: ShopConfig.Item,
    val type: PurchaseType
)

val purchases = mutableListOf<PurchaseEvent>()

fun closeListener(e: InventoryCloseEvent) {
    val purchase = purchases.firstOrNull { it.inv == e.inventory } ?: return
    purchases -= purchase
}

fun baseListener(e: InventoryClickEvent) {
    if (e.inventory == GlobalConfig.main.inventory) {
        mainClick(e)
        return
    }

    val purchase = purchases.firstOrNull { it.inv == e.inventory }
    if (purchase != null) clickInventory(e, purchase)

    val shop = GlobalConfig.shops.firstOrNull { it.inventories.contains(e.inventory) } ?: return
    shopClick(e, shop)
}

fun mainClick(e: InventoryClickEvent) {
    e.isCancelled = true
    if (e.rawSlot > e.inventory.size) return

    val main = GlobalConfig.main
    val p = e.whoClicked as Player

    val block = main.blocks.firstOrNull { it.slot == e.slot } ?: return
    block.commands.forEach {
        p.performCommand(it)
    }
}

fun shopClick(e: InventoryClickEvent, shop: ShopConfig) {
    e.isCancelled = true
    if (e.rawSlot > e.inventory.size) return

    val main = GlobalConfig.main
    val p = e.whoClicked as Player

    //Check Menu Buttons
    if (e.currentItem == main.buttons.menu) {
        p.openInventory(main.inventory)
        return
    }

    if (e.currentItem == main.buttons.forward) {
        val current = shop.inventories.indexOf(e.inventory)
        if (current + 1 == shop.inventories.size) return
        p.openInventory(shop.inventories[current + 1])
        return
    }

    if (e.currentItem == main.buttons.previous) {
        val current = shop.inventories.indexOf(e.inventory)
        if (current == 0) return
        p.openInventory(shop.inventories[current - 1])
    }

    //Buy / Sell Item
    val block = shop.blocks.firstOrNull { it.item == e.currentItem } ?: return
    if (block is ShopConfig.Command) {
        TODO()
    }

    when (e.click) {
        LEFT -> buy(e, p, shop, block as ShopConfig.Item)
        RIGHT -> TODO()
        MIDDLE -> TODO()
        else -> return
    }
}

fun clickInventory(e: InventoryClickEvent, pe: PurchaseEvent) {
    e.isCancelled = true
    val block = pe.block
    val origin = block.item

    val singleCost =
        if (pe.type == PurchaseType.BUY) pe.block.buy / origin.amount
        else pe.block.sell / origin.amount

    val lore =
        if (pe.type == PurchaseType.BUY) GlobalConfig.messages.buyLore
        else GlobalConfig.messages.sellLore

    when (e.currentItem) {
        PurchaseItems.set1 -> {
            pe.item.amount = 1
            pe.item.setLore(lore.mixPlaceholder(price = singleCost))
        }
        PurchaseItems.sub10 -> {
            val now = max(1, pe.item.amount - 10)
            pe.item.amount = now
            pe.item.setLore(lore.mixPlaceholder(price = singleCost * now))
        }
        PurchaseItems.sub1 -> {
            val now = max(1, pe.item.amount - 1)
            pe.item.amount = now
            pe.item.setLore(lore.mixPlaceholder(price = singleCost * now))
        }
        PurchaseItems.add1 -> {
            val now = min(64, pe.item.amount + 1)
            pe.item.amount = now
            pe.item.setLore(lore.mixPlaceholder(price = singleCost * now))
        }
        PurchaseItems.add10 -> {
            val now = min(64, pe.item.amount + 10)
            pe.item.amount = now
            pe.item.setLore(lore.mixPlaceholder(price = singleCost * now))
        }
        PurchaseItems.set64 -> {
            pe.item.amount = 64
            pe.item.setLore(lore.mixPlaceholder(price = singleCost * 64))
        }
    }
    pe.inv.setItem(22, pe.item)
}

val base = Array<ItemStack?>(54) { null }.apply {
    set(18, PurchaseItems.set1)
    set(19, PurchaseItems.sub10)
    set(20, PurchaseItems.sub1)
    set(24, PurchaseItems.add1)
    set(25, PurchaseItems.add10)
    set(26, PurchaseItems.set64)
}

fun Material.prettyName() = name.split("_").joinToString(" ") { it.toLowerCase().capitalize() }

fun buy(e: InventoryClickEvent, p: Player, shop: ShopConfig, block: ShopConfig.Item) {
    val item = e.currentItem.clone()
    if (block.buy == -1.0) return

    val name = if (item.itemMeta.hasDisplayName()) item.itemMeta.displayName else item.type.prettyName()

    val inv = Bukkit.createInventory(null, 54, "&2Buying $name".c).apply {
        contents = base.clone()

        val buyLore = GlobalConfig.messages.buyLore.mixPlaceholder(price = block.buy)
        setItem(22, e.currentItem.setLore(buyLore))
        setItem(39, PurchaseItems.buyConfirm)
        setItem(41, PurchaseItems.cancel)
    }

    p.openInventory(inv)
    purchases.add(PurchaseEvent(inv, item, shop, p, block, PurchaseType.BUY))
}

